||| Render Djot to HTML
module SSG.Djot.Render

import SSG.HTML
import SSG.Djot.Inline
import SSG.Djot.Block

import Data.List
import Data.Nat

||| Render a single inline element as HTML
export
renderInline : Inline -> Maybe (t ** Html t)
-- FIXME: escape text
renderInline (Text text) = Just (_ ** Text text)
renderInline SoftLineBreak = Nothing
renderInline HardLineBreak = Just (_ ** Void "br" [])

||| Render a list of inline elments to html
export
renderInlines : List Inline -> (types ** DList _ Html types)
renderInlines xs = fromList . catMaybes $ map renderInline xs

||| Render a single block element as HTML
export
renderBlock : Block t -> (t ** Html t)

||| Render a list of block level elements to html
export
renderBlocks : {types: _} -> DList _ Block types -> (types' ** DList _ Html types')

renderBlock (Paragraph content) =
  (_ ** Normal "p" []  (snd (renderInlines content)))
renderBlock (Heading 1 content {non_zero = ItIsSucc} {in_range}) =
  (_ ** Normal "h1" [] (snd (renderInlines content)))
renderBlock (Heading 2 content {non_zero = ItIsSucc} {in_range}) =
  (_ ** Normal "h2" [] (snd (renderInlines content)))
renderBlock (Heading 3 content {non_zero = ItIsSucc} {in_range}) =
  (_ ** Normal "h3" [] (snd (renderInlines content)))
renderBlock (Heading 4 content {non_zero = ItIsSucc} {in_range}) =
  (_ ** Normal "h4" [] (snd (renderInlines content)))
renderBlock (Heading 5 content {non_zero = ItIsSucc} {in_range}) =
  (_ ** Normal "h5" [] (snd (renderInlines content)))
renderBlock (Heading 6 content {non_zero = ItIsSucc} {in_range}) =
  (_ ** Normal "h6" [] (snd (renderInlines content)))
renderBlock (Heading (6 + (S n)) content {non_zero = ItIsSucc} {in_range}) =
  absurd in_range

renderBlocks xs = dMap' (\_, x => renderBlock x) xs

||| Render a complete html document from a list of block level elements
export
renderDocument : {types : _} -> DList _ Block types -> Html "html"
renderDocument xs =
  Normal "html" ["lang" =$ "en"] [
    Normal "body" [] (snd (renderBlocks xs))
  ]
